//-------------------------------------------------------------------//
//                         STEROWNIK AVR-USB
//						   Marcin Bieda 2008
//
//						   main.c
//-------------------------------------------------------------------//

#include "usbconfig.h"
#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/pgmspace.h>

#include "usbdrv/usbdrv.h"
#include "usbconfig.h"
#include "krokowy.h"
#include "makra.h"

//--------------------------- Definicje instrukcji wysyanych do uC

#define INS_TEST 0			//test poczenia
#define INS_USTAW_WYJ 1		//bezporednie ustawienie wyj
#define	INS_ODCZYT_SENS 2	//odczyt z czujnikw (sensorw)
#define	INS_TRYB 3			// 0 - cigy obrt; 1 - obrt o okrelony kt
#define	INS_STEROWANIE 4	// 0 - penokrokowy; 1 - pkrokowy; 2 - falowy						
#define INS_KIERUNEK 5		//kierunek obrotu
#define INS_OPOZ 6			//prdko
#define	INS_KROKI 7			//ilo krokw na peen obrt
#define	INS_KAT 8			//kt obrotu
#define INS_SERVO 9			//ustawia serwo


typedef struct SILNIK_KROKOWY_ //struktura danych dot. silnika
{
	uint8_t ster;
	uint8_t tryb;
	uint16_t rpm;
	uint8_t kierunek;
	uint8_t kroki;
	uint8_t kat;
} SILNIK_KROKOWY;

//--------------------------- Inicjalizacja zminennych globalnych

SILNIK_KROKOWY silnik;
static uchar    wyj_bufor_usb[8];


//--------------------------- Deklaracje i definicje Funkcji

uint8_t ustaw_wyjscia (uint8_t stan)
{
	uint8_t a = PORT(KROK_PORT);
	a = a & ~KROK_MASKA;
	PORT(KROK_PORT) = a | (stan<<KROK_PIN);
	return 0;
}


uchar   usbFunctionSetup(uchar wej_bufor_usb[8]) //funkcja obsugujca USB
{
	uint8_t   dlugosc = 0;

    if(wej_bufor_usb[1] == INS_TEST)
	{      
        dlugosc = 2;
        wyj_bufor_usb[0] = wej_bufor_usb[2];
        wyj_bufor_usb[1] = wej_bufor_usb[3];
    }
	else if(wej_bufor_usb[1] == INS_USTAW_WYJ)
	{ 
        dlugosc = 1;
        wyj_bufor_usb[0] = ustaw_wyjscia(wej_bufor_usb[2]);
    }
	else if(wej_bufor_usb[1] == INS_ODCZYT_SENS)
	{ 
        dlugosc = 1;
		wyj_bufor_usb[0] = ((PIN(SENS_PORT_B)>>SENS_PIN_B)&1);
		wyj_bufor_usb[0] = ((PIN(SENS_PORT_A)>>SENS_PIN_A)&1) + (wyj_bufor_usb[0]<<1);
    }
	else if(wej_bufor_usb[1] == INS_TRYB)
	{
		dlugosc = 1;
		silnik.tryb = wej_bufor_usb[2];
		wyj_bufor_usb[0] = 0;
	}
	else if(wej_bufor_usb[1] == INS_STEROWANIE)
	{
		dlugosc = 1;
		silnik.ster = wej_bufor_usb[2];
		wyj_bufor_usb[0] = 0;
	}
	else if(wej_bufor_usb[1] == INS_KIERUNEK)
	{
		dlugosc = 1;
		silnik.kierunek = wej_bufor_usb[2];
		wyj_bufor_usb[0] = 0;
	}
	else if(wej_bufor_usb[1] == INS_OPOZ)
	{
		dlugosc = 1;
		silnik.rpm = wej_bufor_usb[2] | (wej_bufor_usb[3] << 8);
		wyj_bufor_usb[0] = 0;
	}
	else if(wej_bufor_usb[1] == INS_KROKI)
	{
		dlugosc = 1;
		silnik.kroki = wej_bufor_usb[2];
		wyj_bufor_usb[0] = 0;
	}
	else if(wej_bufor_usb[1] == INS_KAT)
	{
		dlugosc = 1;
		silnik.kat = wej_bufor_usb[2];
		wyj_bufor_usb[0] = 0;
	}
	else if(wej_bufor_usb[1] == INS_SERVO)
	{
		dlugosc = 1;
		OCR1A = wej_bufor_usb[2] | (wej_bufor_usb[3] << 8);
		wyj_bufor_usb[0] = 0;
	}
    usbMsgPtr = wyj_bufor_usb;
    return dlugosc;
}



//----------------------------- Funkcja main

int main(void)
{
	uint8_t   i, j;
	uint16_t czas;
	
	DDRD = 0; //wczamy USB 
	
	DDR(KROK_PORT) |= KROK_MASKA;	//Konfiguracja portw
	PORT(KROK_PORT) &= ~KROK_MASKA;
	
    PORTD = 0;
	DDRD=0;	//od strony komputera urzdzenie wyglda na odczone
  //  j = 0;
 //   while(--j){        
 //       i = 0;
 //       while(--i);     
 //   }  //prymitywne opznienie

	 //wczamy USB  

	DDR(KROK_PORT) |= KROK_MASKA;	//Konfiguracja portw
	PORT(KROK_PORT) &= ~KROK_MASKA;
	
	
	TCCR2= 1<<3 | 2; 	//CTC, preskaler /8
	OCR2=150;			//przepenienie co 100us
	DDRB|=1<<1;
	uint8_t xxx=0;
	
	TCCR1A=1<<7|1<<1; 
	TCCR1B=1<<3|1<<4|1<<1; //Fast PWM, preskaler /8
	ICR1=30000; //przebieg 50Hz
	OCR1A=0; //wartoci 1500 - 3000 <- regulacja serva
	
	
	usbInit(); //zainicjowanie USB oraz wczenie obsugi przerwa
    sei();
    
    for(;;) //ptla gwna
	{    
        
        usbPoll(); //obsuga USB
		if(TIFR & (1 << OCF2)) //Jeli mineo 100us
		{
            TIFR = 1 << OCF2;
			czas++;
			if(silnik.rpm==0) czas=1;
			if(czas==silnik.rpm) 
			{
				czas=0;
				if(silnik.tryb)
				{
					if((PIN(SENS_PORT_A)>>SENS_PIN_A)&1)
					{
						if(silnik.kat)
						{
							silnik.kat--;
							krok(&xxx,silnik.ster);
							if(silnik.kierunek)
								xxx++;
							else
								xxx--;
						}
					}				
				}
				else
				{
					krok(&xxx,silnik.ster);
					if(silnik.kierunek)
						xxx++;
					else
						xxx--;
				}
			}
        }
    }
    return 0;
}

/* ------------------------------------------------------------------------- */
