#include "USBfun.h"

usb_dev_handle      *handle;
unsigned char       buffer[BUF_LEN];
char                runTest = 0;
int                 i, nBytes, vendor = USBDEV_VENDOR, device = USBDEV_PRODUCT;

static int  usbGetStringAscii(usb_dev_handle *dev, int index, int langid, char *buf, int buflen)
{
char    buffer[256];
int     rval, i;

    if((rval = usb_control_msg(dev, USB_ENDPOINT_IN, USB_REQ_GET_DESCRIPTOR, (USB_DT_STRING << 8) + index, langid, buffer, sizeof(buffer), 1000)) < 0)
        return rval;
    if(buffer[1] != USB_DT_STRING)
        return 0;
    if((unsigned char)buffer[0] < rval)
        rval = (unsigned char)buffer[0];
    rval /= 2;
    
    for(i=1;i<rval;i++){
        if(i > buflen)  
            break;
        buf[i-1] = buffer[2 * i];
        if(buffer[2 * i + 1] != 0) 
            buf[i-1] = '?';
    }
    buf[i-1] = 0;
    return i-1;
}

int usbOpenDevice(usb_dev_handle **device, int vendor, char *vendorName, int product, char *productName)
{
struct usb_bus      *bus;
struct usb_device   *dev;
usb_dev_handle      *handle = NULL;
int                 errorCode = USB_ERROR_NOTFOUND;
static int          didUsbInit = 0;
wxString a;

    if(!didUsbInit){
        didUsbInit = 1;
        usb_init();
    }
    usb_find_busses();
    usb_find_devices();

    for(bus=usb_get_busses(); bus; bus=bus->next){
        for(dev=bus->devices; dev; dev=dev->next){
            if(dev->descriptor.idVendor == vendor && dev->descriptor.idProduct == product){
                char    string[256];
                int     len;
                handle = usb_open(dev); /* we need to open the device in order to query strings */
                if(!handle){
                    errorCode = USB_ERROR_ACCESS;
                    a << "Warning: cannot open USB device: " << usb_strerror() << "\n";
                    continue;
                }
                if(vendorName == NULL && productName == NULL){  /* name does not matter */
                    break;
                }
                /* now check whether the names match: */
                len = usbGetStringAscii(handle, dev->descriptor.iManufacturer, 0x0409, string, sizeof(string));
                if(len < 0){
                    errorCode = USB_ERROR_IO;
                    a << "Warning: cannot query manufacturer for device: " << usb_strerror() << "\n";
                }else{
                    errorCode = USB_ERROR_NOTFOUND;
                    /* fprintf(stderr, "seen device from vendor ->%s<-\n", string); */
                    if(strcmp(string, vendorName) == 0){
                        len = usbGetStringAscii(handle, dev->descriptor.iProduct, 0x0409, string, sizeof(string));
                        if(len < 0){
                            errorCode = USB_ERROR_IO;
                            a << "Warning: cannot query product for device: " << usb_strerror() << "\n";
                        }else{
                            errorCode = USB_ERROR_NOTFOUND;
                            /* fprintf(stderr, "seen product ->%s<-\n", string); */
                            if(strcmp(string, productName) == 0)
                                break;
                        }
                    }
                }
                usb_close(handle);
                handle = NULL;
            }
        }
        if(handle)
            break;
    }
    if(handle != NULL){
        errorCode = 0;
        *device = handle;
    }
    return errorCode;
}


int test_polaczenia (int ile, usb_dev_handle *handle)
{
    wxString a;
    int i;
    int nBytes;
    unsigned char       buffer[BUF_LEN];
    int j=0;

    a << "Testing connection";
    a << "\n";
    for(i=0;i<ile;i++)
    {
        int v,r;
        v = rand() & 0xffff;
        nBytes = usb_control_msg(handle, USB_TYPE_VENDOR | USB_RECIP_DEVICE | USB_ENDPOINT_IN, CMD_ECHO, v, 0, (char *)buffer, sizeof(buffer), 5000);
        if(nBytes < 2)
        {
            if(nBytes < 0)
            a << "USB error: " << usb_strerror() << "\n";
            a << "only " << nBytes << " bytes received in iteration " << i << "\n";
            j++;

        }
        r = buffer[0] | (buffer[1] << 8);
        if(r != v)
        {
        a << "data error: received " << r << " instead of " << v << " in iteration " << i << "\n";
        j++;
        exit(1);
        }
    }
    a << "test succeeded" << "\n";
    return j;
}

int wyslij_instrukcje (int instrukcja, int dane, usb_dev_handle *handle)
{
    int ile;
    unsigned char       buffer[BUF_LEN];
    ile = usb_control_msg(handle, USB_TYPE_VENDOR | USB_RECIP_DEVICE | USB_ENDPOINT_IN, instrukcja, dane, 0, (char *)buffer, sizeof(buffer), 5000);
    if(ile!=1)
       return -1;

    return buffer[0];
}
